/************************************************************************************/
/*                            DRAM Controller fr AVRs                              */
/*                                                                                  */
/* Mit diesen Funktionen lsst sich ein 256kB bis 4096kB DRAM (Module) DRAM an		*/
/* inem AVR verwenden. Die Adressierung ist dabei linear in Bytes.				  	*/
/* Mit den Funktionen readbyte, writebyte bzw. readstring und writestring werden 	*/
/* Daten geschrieben bzw. gelesen.													*/
/* Die string Funktionen verwenden den Page Modus zum Schreiben und Lesen, so dass	*/
/* durchschnittlich etwa 1MByte/s Datenrate erreicht werden.						*/
/*                                                                                  */
/* Der Refresh des Speichers geschieht im Hintergrund mittels Timer 0. Die Refresh  */
/* Routine bentigt bei 18,432MHz etwa 5% der CPU Leistung.                         */
/* Da fr den Refresh sehr viel Strom bentigt wird (kurzzeitig steigt die          */
/* Stromaufnahme des DRAMs von etwa 20-40mA auf ber 100mA an), sollte in           */
/* Speichernhe ein Kondensator mit mindestens einigen 10uF eingebaut werden.		*/
/*                                                                                  */
/* In der Datei DRAMIO.h werden die DRAM Parameter wie Anschlussbelegung, Gre,    */
/* Speicherorganisation und sonstige Einstellungen festgelegt:                      */
/*                                                                                  */
/* #define DRAMPORT		PORTA	Legt den verwendeten Port fr die Datenleitungen    */
/*                              (und bei DRAMs mit OE\ die Adressleitungen) fest.   */
/* #define DRAMADR		PORTC	Legt den verwendeten Port fr die Adressleitungen   */
/*                              bei DRAMs ohne OE\ fest.  						 	*/
/* #define DRAMCONTROL	PORTD	Legt den verwendeten Port fr die Steuerleitungen   */
/*                              und Adressleitungen (A8 aufwrts) fest. 		 	*/
/* #define A8          	0		Pin an den A8 angeschlossen wird.					*/
/* #define A9          	1		Pin an den A9 angeschlossen wird.					*/
/* #define A10         	2		Pin an den A10 angeschlossen wird.					*/
/* #define RAS         	3		Pin an den RAS angeschlossen wird.					*/
/* #define CASL        	4		Pin an den CAS0 (LCAS) angeschlossen wird.			*/
/* #define CASH        	5		Pin an den CAS1 (UCAS, bzw. 2. Baustein).			*/
/*								angeschlossen wird (falls vorhanden)				*/		
/* #define WE         	6		Pin an den WE angeschlossen wird.					*/
/* #define OE         	7		Pin an den OE angeschlossen wird (falls vorhanden).	*/
/* 																					*/
/* #define DRAMBITS    	8		Legt die Breite eines Datenworts am DRAM fest.		*/
/* 								Hat der DRAM 1 CAS Anschluss, dann steht hier 8		*/
/*								Hat er 2, oder es werden 2 DRAMs verwendet, dann	*/
/*								steht hier 16.										*/
/* #define DRAMWORD    4096		Legt die Gre des DRAMs fest. Hat ein DRAM z.B. 	*/
/* 								die Gre 512kx16 dann wird hier 512 angegeben.		*/
/* #define DRAMSIZE    			Gibt die Gre des DRAMs in Byte an. Dieser Wert 	*/
/* 								wird aus DRAMWORD und DRAMBITs berechnet und muss	*/
/*								daher normalerweise nicht angegeben werden.			*/
/* 																					*/
/* #define ReadRefresh 1    	Gibt an, ob whrend eines Speicherrefreshs Daten	*/
/* 								aus dem DRAM gelesen/ins DRAM geschrieben werden 	*/
/*								knnen. Dazu wird der Refresh unterbrochen, die		*/
/*								Datenbertragung ausgefhrt und der Refresh fort-	*/
/*								gesetzt. Allerdings wird dadurch alles ein klein 	*/
/*								wenig langsamer. Falls also mehr Geschwindigkeit	*/
/*								bentigt wird, kann man hier was optimieren .		*/
/************************************************************************************/


#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/pgmspace.h>
#include "dram.h"
#include "dramIO.h"
#include "uart.h"

#if (ReadRefresh==1)
	volatile unsigned char Refresh;
#endif


unsigned char mem_init(void)
{   unsigned long i;
	unsigned char j, temp[13];
	unsigned char muster[13]={13,150,170,85,254,130,5,134,0,127,160,234,25};

    TCCR0=4;                    //Timer0: 72kHz
    TIMSK=(1<<TOV0);                    //Timer Interrupt: 281Hz fr Refresh

    DRAMCONTROL=255;

        asm volatile("clr __tmp_reg__" "\n"				//DRAMPorts = Ausgang
                     "dec __tmp_reg__" "\n"
                     "out %0-1, __tmp_reg__ " "\n\t" 
                     "out %1-1, __tmp_reg__ " "\n\t"
                     "out %2-1, __tmp_reg__ " "\n\t"
                      : 
                      : "I" (_SFR_IO_ADDR(DRAMCONTROL)),"I" (_SFR_IO_ADDR(DRAMPORT)),"I" (_SFR_IO_ADDR(DRAMADR))
                      );


    for (i=0;i<(long)1024*DRAMSIZE-13;i+=13)		//Speichertest
        write_string(i, muster, 13);				//13Byte Teststring in RAM schreiben

    for (i=0;i<(long)1024*DRAMSIZE-13;i+=13)		//Speichertest
		{
        read_string(i, temp, 13);					//13Bytes aus RAM lesen
		for (j=0; j<13; j++)
	        if (temp[j]!=muster[j]) 				//Teststring mit Referenz vergleichen
    	        return 1;
	  	}
    return 0;}

void writeMem(void)
{
  	unsigned long i;
	unsigned int c;
    for (i=0;i<(long)1024L*DRAMSIZE;i++)
		{
	  	while ((c = uart_getc()) & UART_NO_DATA);
        write_byte(i, (char)c&255);
		}
}

void readMem(void)
{
  	unsigned long i;
    for (i=0;i<(long)1024L*DRAMSIZE;i++)
		{
	  	uart_putc(read_byte(i));
		}
}


